
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/ol_concourse:ol-concourse

from setuptools import setup

setup(**{
    'install_requires': (
        'bridge==0.2.0',
        'ol-concourse-pipelines==0.1.0',
        'pydantic<3.0.0,>=2',
    ),
    'name': 'ol-concourse',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'ol_concourse',
        'ol_concourse.lib',
        'ol_concourse.lib.jobs',
        'ol_concourse.lib.models',
    ),
    'python_requires': '>=3.12',
    'version': '0.5.6',
})
